'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var styled = require('styled-components');
var styled__default = _interopDefault(styled);

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

const pivot = ({
  showPivot
}) => styled.css`
  position: absolute;
  left: 50%;
  top: 50%;
  transform-origin: 50%;
  transform-style: preserve-3d;

  ${showPivot && styled.css`
      box-shadow: 0 0 5px 5px yellow;
    `};
`;

const SceneContext = React.createContext();
/*
  isWebkit - Used to apply a hidden drop-shadow filter in webkit browsers only.
  For some reason this prevents depth fighting issues where faces flicker in
  webkit browsers but causes depth fighting to happen in non-webkit browsers 🤷.
*/

const isWebkit = /(webkit)/.test(navigator.userAgent.toLowerCase());
const Container = styled__default.div`
  ${({
  ambientColor,
  shadow
}) => styled.css`
    ${pivot}
    perspective: ${({
  perspective
}) => perspective + 'px'};

    ${isWebkit && styled.css`
        filter: drop-shadow(
          0 8px 8px
            rgba(${ambientColor.r}, ${ambientColor.g}, ${ambientColor.b}, ${shadow ? 0.5 : 0})
        );
      `};
  `}
`;

function Scene({
  ambientColor,
  shaded,
  occluded,
  perspective,
  ...rest
}) {
  return React__default.createElement(SceneContext.Provider, {
    value: {
      ambientColor,
      shaded,
      occluded
    }
  }, React__default.createElement(Container, _extends({
    ambientColor: ambientColor,
    perspective: perspective
  }, rest)));
}

Scene.defaultProps = {
  ambientColor: {
    r: 0,
    g: 0,
    b: 0
  },
  occluded: {
    size: 0,
    strength: 0
  },
  perspective: 2000,
  shaded: {
    strength: 0
  }
};

function useScene() {
  return React.useContext(SceneContext);
}

const transform = ({
  rotation,
  position
}) => {
  let transforms = [];

  if (position) {
    if (position.x) {
      transforms.push(`translateX(${position.x})`);
    }

    if (position.y) {
      transforms.push(`translateY(${position.y})`);
    }

    if (position.z) {
      transforms.push(`translateZ(${position.z})`);
    }
  }

  if (rotation) {
    if (rotation.x) {
      transforms.push(`rotateX(${rotation.x}deg)`);
    }

    if (rotation.y) {
      transforms.push(`rotateY(${rotation.y}deg)`);
    }

    if (rotation.z) {
      transforms.push(`rotateZ(${rotation.z}deg)`);
    }
  }

  return styled.css`
    transform: ${transforms.join(' ')};
  `;
};

const Group = styled__default.div`
  ${pivot}
  ${transform}
`;

const texture = ({
  ambientColor,
  occlusion,
  occluded,
  shaded,
  texture
}) => {
  const images = [];
  const sizes = [];
  const positions = [];

  if (occlusion.includes('top')) {
    images.push(`linear-gradient(to bottom, rgba(${ambientColor.r}, ${ambientColor.g}, ${ambientColor.b}, ${occluded.strength}) 0%, rgba(${ambientColor.r}, ${ambientColor.g}, ${ambientColor.b}, 0) ${occluded.size}px)`);
    positions.push('center');
    sizes.push('100%');
  }

  if (occlusion.includes('bottom')) {
    images.push(`linear-gradient(to top, rgba(${ambientColor.r}, ${ambientColor.g}, ${ambientColor.b}, ${occluded.strength}) 0%, rgba(${ambientColor.r}, ${ambientColor.g}, ${ambientColor.b}, 0) ${occluded.size}px)`);
    positions.push('center');
    sizes.push('100%');
  }

  if (occlusion.includes('left')) {
    images.push(`linear-gradient(to right, rgba(${ambientColor.r}, ${ambientColor.g}, ${ambientColor.b}, ${occluded.strength}) 0%, rgba(${ambientColor.r}, ${ambientColor.g}, ${ambientColor.b}, 0) ${occluded.size}px)`);
    positions.push('center');
    sizes.push('100%');
  }

  if (occlusion.includes('right')) {
    images.push(`linear-gradient(to left, rgba(${ambientColor.r}, ${ambientColor.g}, ${ambientColor.b}, ${occluded.strength}) 0%, rgba(${ambientColor.r}, ${ambientColor.g}, ${ambientColor.b}, 0) ${occluded.size}px)`);
    positions.push('center');
    sizes.push('100%');
  }

  if (shaded && texture.shade) {
    images.push(`linear-gradient(rgba(${ambientColor.r}, ${ambientColor.g}, ${ambientColor.b}, ${texture.shade * shaded.strength}) 0%, rgba(${ambientColor.r}, ${ambientColor.g}, ${ambientColor.b}, ${texture.shade * shaded.strength}) 100%)`);
    positions.push('center');
    sizes.push('100%');
  }

  if (texture.image) {
    images.push(`url(${texture.image})`);
    positions.push(`${texture.position || 'center'}`);
    sizes.push(`${texture.size || '100%'}`);
  }

  return styled.css`
    background-color: ${texture.color || '#d8730b'};
    background-image: ${images.join(', ')};
    background-position: ${positions.join(', ')};
    background-size: ${sizes.join(', ')};
    clip-path: ${texture.clipPath};
    image-rendering: ${texture.filter === 'pixelated' && 'crisp-edges'}; /* Firefox */
    image-rendering: ${texture.filter === 'pixelated' && 'pixelated'};
  `;
};

const Container$1 = styled__default.div`
  ${({
  ambientColor,
  occluded,
  size,
  shadow,
  showBackface
}) => styled.css`
    ${transform};
    height: ${size.y}px;
    width: ${size.x}px;
    position: absolute;
    transform-style: preserve-3d;

    ${shadow && styled.css`
        &:before {
          content: '';
          backface-visibility: ${showBackface ? 'visible' : 'hidden'};
          box-shadow: 0 0 ${occluded.size}px ${occluded.size / 2}px
            rgba(
              ${ambientColor.r},
              ${ambientColor.g},
              ${ambientColor.b},
              ${occluded.strength}
            );
          transform: rotateY(180deg);
          position: absolute;
          top: 0;
          left: 0;
          right: 0;
          bottom: 0;
        }
      `};

    &:after {
      ${texture};
      content: '';
      backface-visibility: ${showBackface ? 'visible' : 'hidden'};
      position: absolute;
      top: 0;
      left: 0;
      right: 0;
      bottom: 0;
    }
  `}
`;

function Plane({ ...rest
}) {
  const {
    ambientColor,
    occluded,
    shaded
  } = useScene();
  return React__default.createElement(Container$1, _extends({
    ambientColor: ambientColor,
    occluded: occluded,
    shaded: shaded
  }, rest));
}

Plane.defaultProps = {
  occlusion: [],
  position: {
    x: '-50%',
    y: '-50%'
  },
  size: {
    x: 128,
    y: 128
  }
};

function Box({
  culled,
  pivotPosition: pivotPositionOverride,
  reversed,
  shadows: consumerShadows,
  size: consumerSize,
  texture,
  textureBack,
  textureBottom,
  textureFront,
  textureLeft,
  textureRight,
  textureTop,
  ...rest
}) {
  const {
    shaded,
    occluded
  } = useScene();
  const planes = ['front', 'back', 'top', 'bottom', 'left', 'right'].filter(plane => !culled.includes(plane));

  function getOccludedEdges(array) {
    return array.map(item => {
      if (!occluded) return false;

      if (reversed) {
        return planes.includes(item.side) ? item.edge : false;
      } else {
        return consumerShadows.includes(item.side) ? item.edge : false;
      }
    }).filter(edge => edge);
  }

  const size = typeof consumerSize === 'object' ? consumerSize : {
    x: consumerSize,
    y: consumerSize,
    z: consumerSize
  };
  const pivotPosition = {
    x: 'center',
    y: 'center',
    z: 'center',
    ...pivotPositionOverride
  };
  const pivotDifferenceX = {
    start: -size.x / 2,
    center: 0,
    end: size.x / 2
  }[pivotPosition.x];
  const pivotDifferenceY = {
    start: -size.y / 2,
    center: 0,
    end: size.y / 2
  }[pivotPosition.y];
  const pivotDifferenceZ = {
    start: -size.z / 2,
    center: 0,
    end: size.z / 2
  }[pivotPosition.z];
  const occlusions = {
    front: getOccludedEdges([{
      edge: 'top',
      side: 'top'
    }, {
      edge: 'bottom',
      side: 'bottom'
    }, {
      edge: 'left',
      side: 'left'
    }, {
      edge: 'right',
      side: 'right'
    }]),
    back: getOccludedEdges([{
      edge: 'top',
      side: 'top'
    }, {
      edge: 'bottom',
      side: 'bottom'
    }, {
      edge: 'left',
      side: 'left'
    }, {
      edge: 'right',
      side: 'right'
    }]),
    top: getOccludedEdges([{
      edge: 'top',
      side: 'back'
    }, {
      edge: 'bottom',
      side: 'front'
    }, {
      edge: 'left',
      side: reversed ? 'right' : 'left'
    }, {
      edge: 'right',
      side: reversed ? 'left' : 'right'
    }]),
    bottom: getOccludedEdges([{
      edge: 'top',
      side: 'front'
    }, {
      edge: 'bottom',
      side: 'back'
    }, {
      edge: 'left',
      side: reversed ? 'right' : 'left'
    }, {
      edge: 'right',
      side: reversed ? 'left' : 'right'
    }]),
    right: getOccludedEdges([{
      edge: 'top',
      side: 'top'
    }, {
      edge: 'bottom',
      side: 'bottom'
    }, {
      edge: 'left',
      side: reversed ? 'back' : 'front'
    }, {
      edge: 'right',
      side: reversed ? 'front' : 'back'
    }]),
    left: getOccludedEdges([{
      edge: 'top',
      side: 'top'
    }, {
      edge: 'bottom',
      side: 'bottom'
    }, {
      edge: 'left',
      side: reversed ? 'front' : 'back'
    }, {
      edge: 'right',
      side: reversed ? 'back' : 'front'
    }])
  };
  const positions = {
    front: {
      x: `calc(-50% + ${pivotDifferenceX}px)`,
      y: `calc(-50% + ${pivotDifferenceY}px)`,
      z: `${size.z / 2 + pivotDifferenceZ}px`
    },
    back: {
      x: `calc(-50% + ${pivotDifferenceX}px)`,
      y: `calc(-50% + ${pivotDifferenceY}px)`,
      z: `-${size.z / 2 - pivotDifferenceZ}px`
    },
    top: {
      x: `calc(-50% + ${pivotDifferenceX}px)`,
      y: `calc(-50% - ${size.y / 2 - pivotDifferenceY}px)`,
      z: `${pivotDifferenceZ}px`
    },
    bottom: {
      x: `calc(-50% + ${pivotDifferenceX}px)`,
      y: `calc(-50% + ${size.y / 2 + pivotDifferenceY}px)`,
      z: `${pivotDifferenceZ}px`
    },
    right: {
      x: `calc(-50% + ${size.x / 2 + pivotDifferenceX}px)`,
      y: `calc(-50% + ${pivotDifferenceY}px)`,
      z: `${pivotDifferenceZ}px`
    },
    left: {
      x: `calc(-50% - ${size.x / 2 - pivotDifferenceX}px)`,
      y: `calc(-50% + ${pivotDifferenceY}px)`,
      z: `${pivotDifferenceZ}px`
    }
  };
  const rotations = {
    front: {
      x: 0,
      y: reversed ? 180 : 0,
      z: 0
    },
    back: {
      x: 0,
      y: reversed ? 0 : 180,
      z: 0
    },
    top: {
      x: 90,
      y: reversed ? 180 : 0,
      z: 0
    },
    bottom: {
      x: 270,
      y: reversed ? 180 : 0,
      z: 0
    },
    right: {
      x: 0,
      y: reversed ? 270 : 90,
      z: 0
    },
    left: {
      x: 0,
      y: reversed ? 90 : 270,
      z: 0
    }
  };
  const shadows = {
    front: !reversed && consumerShadows.includes('front'),
    back: !reversed && consumerShadows.includes('back'),
    top: !reversed && consumerShadows.includes('top'),
    bottom: !reversed && consumerShadows.includes('bottom'),
    left: !reversed && consumerShadows.includes('left'),
    right: !reversed && consumerShadows.includes('right')
  };
  const sizes = {
    front: {
      x: size.x,
      y: size.y
    },
    back: {
      x: size.x,
      y: size.y
    },
    top: {
      x: size.x,
      y: size.z
    },
    bottom: {
      x: size.x,
      y: size.z
    },
    right: {
      x: size.z,
      y: size.y
    },
    left: {
      x: size.z,
      y: size.y
    }
  };
  const textures = {
    front: { ...texture,
      ...textureFront,
      shade: shaded ? reversed ? 0.6 : 0.2 : 0
    },
    back: { ...texture,
      ...textureBack,
      shade: shaded ? reversed ? 0.2 : 0.6 : 0
    },
    top: { ...texture,
      ...textureTop,
      shade: shaded ? reversed ? 0.4 : 0 : 0
    },
    bottom: { ...texture,
      ...textureBottom,
      shade: shaded ? reversed ? 0 : 0.4 : 0
    },
    right: { ...texture,
      ...textureRight,
      shade: shaded ? reversed ? 0.4 : 0.4 : 0
    },
    left: { ...texture,
      ...textureLeft,
      shade: shaded ? reversed ? 0.4 : 0.4 : 0
    }
  };
  return React__default.createElement(Group, rest, planes.map(plane => React__default.createElement(Plane, {
    key: plane,
    occlusion: occlusions[plane],
    position: positions[plane],
    rotation: rotations[plane],
    shadow: shadows[plane],
    size: sizes[plane],
    texture: textures[plane]
  })));
}

Box.defaultProps = {
  culled: [],
  shadows: [],
  size: 128
};

exports.Box = Box;
exports.Group = Group;
exports.Plane = Plane;
exports.Scene = Scene;
exports.useScene = useScene;
